require 'spec_helper'

describe RouteNGNClient::Model do

  before do
    RouteNGNClient.connect(:uri => 'http://127.0.0.1')

    bodyc = '{"account":{}}'
    stub_request(:get, /^http:\/\/127\.0\.0\.1\/system_admin\/accounts\/foobar\.json.*/).to_return(:body => bodyc,
                                                                                                   :status => 200,
                                                                                                   :headers => { 'Content-Type' => 'application/json; charset=utf-8' })


    bodyd = '{"total_count":1,"current_page":1,"per_page":200,"offset":0,"total_pages":1,"carriers":[]}'
    stub_request(:get, /^http:\/\/127\.0\.0\.1\/metallic\/carriers\/foobar\.json.*/).to_return(:body => bodyd,
                                                                                               :status => 200,
                                                                                               :headers => { 'Content-Type' => 'application/json; charset=utf-8' })
  end

  it "should create a new instance given valid attributes" do
    m = Model.new
    m.should be_an_instance_of(Model)
  end

  describe '#to_hash' do
    it "should return an instace of Hash" do
      m = Model.new(:test => 'foo')
      m.to_hash.should be_an_instance_of(Hash)
    end
  end

  describe '#to_json' do
    it "should return an instace of String" do
      m = Model.new(:test => 'foo')
      m.to_json.should be_an_instance_of(String)
    end
  end

  describe '#from_hash' do
    it "should return an instace of Model" do
      m = Model.from_hash(:test => 'foo')
      m.should be_an_instance_of(Model)
    end
  end

  describe '#belongs_to' do
    it "should set parents and accessors" do
      c = Class.new(Model)
      c.belongs_to :account
      c.parents.length.should == 1
      c.parents.first.should be_an_instance_of(RouteNGNClient::Account.class)
      m = c.from_hash(:test => 'foo', :account_id => 'foobar')
      m.should respond_to(:account)
      m.account.should be_an_instance_of(Account)
    end
  end

  describe '#has_many' do
    it "should set children and accessors" do
      c = Class.new(Model)
      c.has_many :carriers
      c.children.length.should == 1
      c.children.first.should be_an_instance_of(RouteNGNClient::Carrier.class)
      m = c.from_hash(:test => 'foo', :carrier_id => 'foobar')
      m.should respond_to(:carriers)
      m.carriers.should be_an_instance_of(Array)
    end
  end

  describe '#has_one' do
    it "should set child and accessors" do
      c = Class.new(Model)
      c.has_one :account
      c.only_children.length.should == 1
      c.only_children.first.should be_an_instance_of(RouteNGNClient::Account.class)
      m = c.from_hash(:test => 'foo', :account_id => 'foobar')
      m.should respond_to(:account)      
      m.account.should be_an_instance_of(Account)
    end
  end

end