require 'spec_helper'

describe RouteNGNClient::Contact do

  it "can be instanstiated" do
    c = Contact.new
    c.should be_an_instance_of(Contact)
  end

  it "can build uri" do
    sip_request = RSIP::Message.parse(INVITE)
    route = Route.new(:outbound_end_point_group => OEPG1)
    oep = OEPG1.end_points.first
    contact = Contact.new :route => route, :end_point => oep

    uri = contact.build_uri(INVITE_SIP_REQUEST1)
    uri.should be_an_instance_of(RSIP::Message::SIPURI)
    uri.user.dialcode.should == "50246380762"
    uri.user.params.should be_empty
    uri.params.should == {"otg"=>"3", "npdi"=>"yes", "rn"=>"2123334545"}
  end

  it "can build aor" do
    sip_request = RSIP::Message.parse(INVITE)
    route = Route.new(:outbound_end_point_group => OEPG1)
    oep = OEPG1.end_points.first
    contact = Contact.new :route => route, :end_point => oep

    aor = contact.build_aor(INVITE_SIP_REQUEST1, IEPG1)
    aor.should be_an_instance_of(String)
  end

  it "should URI.encode the aor" do
    sip_request = RSIP::Message.parse(INVITE)
    oepg2 = OEPG1.clone
    oepg2.attributes.dialcode_template = "123\#${dialcode_match}"
    oep = oepg2.end_points.first
    route = Route.new(:outbound_end_point_group => oepg2)
    contact = Contact.new :route => route, :end_point => oep

    aor = contact.build_aor(sip_request, IEPG1)
    aor.should == '<sip:123%2350246380762@1.1.1.1:5060;otg=3;npdi=yes;rn=2123334545>'
  end

  it "should put the user params in the correct place" do
    sip_request = RSIP::Message.parse(INVITE2)
    oepg1 = OEPG1.clone
    oepg1.attributes.dialcode_template = "123\#${dialcode_match}"
    oep = oepg1.end_points.first
    route = Route.new(:outbound_end_point_group => oepg1)
    contact = Contact.new :route => route, :end_point => oep

    aor = contact.build_aor(sip_request, IEPG1)
    aor.should == '<sip:123%237772223333;npdi=yes;rn=2123334545;otg=4@1.1.1.1:5060>'
  end

end