require 'spec_helper'

describe RouteNGNClient::OutboundEndPointGroup do

  before do
    RouteNGNClient.connect(:uri => 'http://127.0.0.1', :api_key => 'foobar')
    body = '{"total_count":1,"current_page":1,"per_page":25,"offset":0,"total_pages":1,"outbound_end_point_groups":[{"outbound_end_point_group":{"id":"513538546f1cc9a140000016","carrier_id":"513538546f1cc9a140000017","color_code":"#f379de","created_at":"2013-03-05 00:12:04 UTC","default_route_weight":null,"destination_billing_type":"DN","dialcode_expression":"([0-9]*)$","dialcode_template":"${dialcode_match}","end_points":[{"_id":"513538546f1cc9a140000018","port":5060,"host":"1.1.1.1","priority":null}],"max_contact_end_points":null,"name":"OEPG 1","rate_link_types":["dialcode"],"route_table_ids":["513538666f1cc9a14000001b"],"updated_at":"2013-03-05 00:12:04 UTC","uri_params":"","uri_params_blacklist":[],"uri_user_params":"","uri_user_params_blacklist":[]}}]}'
    stub_request(:get, "http://127.0.0.1/metallic/outbound_end_point_groups.json?api_key=foobar&env%5Baccount_id%5D=foofoo3&include_paging_info=true&page=1&per_page=50").to_return(:body => body,
                                                                                                        :status => 200,
                                                                                                        :headers => { 'Content-Type' => 'application/json; charset=utf-8' })
  end

  it "can get all via callback" do
    oepgs = nil

    RouteNGNClient::OutboundEndPointGroup.all({}, :account_id => 'foofoo3') do |models|
      oepgs = models
    end

    oepgs.length.should == 1
    oepgs.first.should be_an_instance_of RouteNGNClient::OutboundEndPointGroup
  end

  it "can get all via fibers" do
    oepgs = nil

    oepgs = RouteNGNClient::OutboundEndPointGroup.all({}, :account_id => 'foofoo3')

    oepgs.length.should == 1
    oepgs.first.should be_an_instance_of RouteNGNClient::OutboundEndPointGroup
  end

  it "can prioritize end_points based on priorities" do
    h = OEPG1.to_hash
    h[:end_points] += [EndPoint.new(:host => '2.2.2.2', :priority => 7), EndPoint.new(:host => '3.3.3.3', :priority => 5)]
    oepg = OutboundEndPointGroup.new h

    eps = oepg.prioritized_end_points
    eps[0].attributes.host.should == '3.3.3.3'
    eps[1].attributes.host.should == '2.2.2.2'
    eps[2].attributes.host.should == '1.1.1.1'
  end

  it "should randomize end_points with the same priorities" do
    h = OEPG1.to_hash
    h[:end_points] = [EndPoint.new(:host => '2.2.2.2', :priority => 5), EndPoint.new(:host => '3.3.3.3', :priority => 5), EndPoint.new(:host => '1.1.1.1', :priority => 5)]
    oepg = OutboundEndPointGroup.new h
    a = 10.times.to_a.map { |i| oepg.prioritized_end_points }
    a.select { |x| x.map { |ep| ep.attributes.host } != a.first.map { |ep| ep.attributes.host } }.length.should > 0 #some should have different orderings
  end

end