require 'spec_helper'

describe RouteNGNClient::Route do

  before do
    RouteNGNClient.connect(:uri => 'http://127.0.0.1')
  end

  let(:iepg1) {
    InboundEndPointGroup.new({ "name"=>"IEPG 1",
                               "contact_template"=>"<sip:${ruri.dialcode}${ruri.user_params}${oepg.uri_user_params}@${oep.host}${oep.port}${ruri.params}${oepg.uri_params}>",
                               "multi_line_contacts"=>true,
                               "color_code"=>"#a4f379",
                               "route_on_type"=>"DN",
                               "min_percent_price_margin"=>nil,
                               "min_absolute_price_margin"=>nil,
                               "min_route_weight"=>nil,
                               "dialcode_expression"=>"([0-9]*)$",
                               "dialcode_prefix_expression"=>"^(.*D+)",
                               "ani_expression"=>"([0-9]*)$",
                               "ani_prefix_expression"=>"^(.*D+)",
                               "destination_billing_type"=>"DN",
                               "external_lrn_lookup_enabled"=>false,
                               "created_at"=>"2013-03-05T00:11:29Z",
                               "updated_at"=>"2013-03-05T00:35:24Z",
                               "id"=>"513538316f1cc9a140000010",
                               "carrier_id"=>"513538316f1cc9a140000011",
                               "primary_route_table_id"=>"513538666f1cc9a14000001b",
                               "secondary_route_table_id"=>"",
                               "allow_list" => {"id"=>"513538316f1cc9a14000000e",
                                 "uri_matchers" => [{"header"=>"request",
                                                    "prefix"=>"",
                                                    "user_params"=>"tgrp=foo",
                                                    "uri_params"=>"",
                                                    "created_at"=>"2013-03-05T00:11:29Z",
                                                    "updated_at"=>"2013-03-05T00:11:29Z",
                                                    "id"=>"513538316f1cc9a14000000f"}]},
                               "end_points"=> [{"end_point"=> {"host"=>"127.0.0.1",
                                                               "port"=>5060,
                                                               "priority"=>nil,
                                                               "created_at"=>nil,
                                                               "updated_at"=>nil,
                                                               "id"=>"51353dcc6f1cc9f910000006"}}]
                              })
  }

  let(:oepg1) {
    OutboundEndPointGroup.new({
                               "id"=>"513538546f1cc9a140000016",
                               "carrier_id"=>"513538546f1cc9a140000017",
                               "color_code"=>"#f379de",
                               "created_at"=>"2013-03-05 00:12:04 UTC",
                               "default_route_weight"=>nil,
                               "destination_billing_type"=>"DN",
                               "dialcode_expression"=>"([0-9]*)$",
                               "dialcode_template"=>"${dialcode_match}",
                               "end_points"=>
                                [{"_id"=>"513538546f1cc9a140000018",
                                  "port"=>5060,
                                  "host"=>"1.1.1.1",
                                  "priority"=>nil}],
                               "max_contact_end_points"=>nil,
                               "name"=>"OEPG 1",
                               "rate_link_types"=>["dialcode"],
                               "route_table_ids"=>["513538666f1cc9a14000001b"],
                               "updated_at"=>"2013-03-05 00:12:04 UTC",
                               "uri_params"=>"",
                               "uri_params_blacklist"=>[],
                               "uri_user_params"=>"abc=123",
                               "uri_user_params_blacklist"=>[]
                              })
  }

  let(:outbound_rate1) {
    OutboundRate.new({
                      "id"=>"51353a256f1cc98978000007",
                      "price"=>0.572,
                      "type"=>"inter",
                      "effective_at"=>"2013-03-05 00:12:37 UTC",
                      "created_at"=>"2013-03-05 00:19:49 UTC",
                      "updated_at"=>"2013-03-05 00:19:49 UTC",
                      "line_number"=>1,
                      "dialcode"=>"3217689",
                      "account_id"=>"5135354e6f1cc9a140000004",
                      "rate_sheet_id"=>"5135388c6f1cc9a14000001f",
                      "outbound_end_point_group_id"=>"513538546f1cc9a140000016"
                    })
  }

  let(:oepg2) {
    OutboundEndPointGroup.new({
                               "id"=>"513538546f1cc9a140000028",
                               "carrier_id"=>"513538546f1cc9a140000017",
                               "color_code"=>"#f379de",
                               "created_at"=>"2013-03-05 00:12:04 UTC",
                               "default_route_weight"=>nil,
                               "destination_billing_type"=>"DN",
                               "dialcode_expression"=>"([0-9]*)$",
                               "dialcode_template"=>"${dialcode_match}",
                               "end_points"=>
                                [{"_id"=>"513538546f1cc9a140000019",
                                  "port"=>5060,
                                  "host"=>"2.2.2.2",
                                  "priority"=>nil}],
                               "max_contact_end_points"=>nil,
                               "name"=>"OEPG 1",
                               "rate_link_types"=>["dialcode"],
                               "route_table_ids"=>["513538666f1cc9a14000001b"],
                               "updated_at"=>"2013-03-05 00:12:04 UTC",
                               "uri_params"=>"",
                               "uri_params_blacklist"=>[],
                               "uri_user_params"=>"",
                               "uri_user_params_blacklist"=>[]
                              })
  }

  let(:outbound_rate2) {
    OutboundRate.new({
                      "id"=>"51353a256f1cc98973234320000f",
                      "price"=>0.9,
                      "type"=>"inter",
                      "effective_at"=>"2013-03-05 00:12:37 UTC",
                      "created_at"=>"2013-03-05 00:19:49 UTC",
                      "updated_at"=>"2013-03-05 00:19:49 UTC",
                      "line_number"=>1,
                      "dialcode"=>"3217689",
                      "account_id"=>"5135354e6f1cc9a140000004",
                      "rate_sheet_id"=>"5135388c6f1cc9a14000001f",
                      "outbound_end_point_group_id"=>"513538546f1cc9a140000028"
                    })
  }

  it "can be instantiated" do
    Route.new.should be_an_instance_of(Route)
  end

  it "can return contacts" do
    route = Route.new(:outbound_rate => outbound_rate1, :outbound_end_point_group => oepg1)
    sip_request = RSIP::Message.parse(INVITE)
    a = route.contacts
    a.should be_an_instance_of(Array)
    a.length.should == 1
    c = a.first
    c.should be_an_instance_of(Contact)
    aor = c.build_aor(sip_request, iepg1)
    aor.should == '<sip:50246380762;abc=123@1.1.1.1:5060;otg=3;npdi=yes;rn=2123334545>'
  end

  it "can build contacts with inbound host and inbound port" do
    iepg2 = iepg1.dup
    iepg2.attributes.contact_template = "<sip:${ruri.dialcode}${ruri.user_params}${oepg.uri_user_params}@${iep.host}${iep.port}${ruri.params}${oepg.uri_params}>"
    route = Route.new(:outbound_rate => outbound_rate1, :outbound_end_point_group => oepg1)
    sip_request = RSIP::Message.parse(INVITE)
    a = route.contacts
    a.should be_an_instance_of(Array)
    a.length.should == 1
    c = a.first
    c.should be_an_instance_of(Contact)
    aor = c.build_aor(sip_request, iepg1, :inbound_end_point_host => '2.2.2.2', :inbound_end_point_port => 5060)
    aor.should == '<sip:50246380762;abc=123@2.2.2.2:5060;otg=3;npdi=yes;rn=2123334545>'
  end

  it "can build contacts with outbound rate" do
    iepg2 = iepg1.dup
    iepg2.attributes.contact_template = "<sip:${ruri.dialcode}${ruri.user_params}${oepg.uri_user_params}@${iep.host}${iep.port}${ruri.params}${oepg.uri_params};or_price=${or.price}>"
    route = Route.new(:outbound_rate => outbound_rate1, :outbound_end_point_group => oepg1)
    sip_request = RSIP::Message.parse(INVITE)
    a = route.contacts
    a.should be_an_instance_of(Array)
    a.length.should == 1
    c = a.first
    c.should be_an_instance_of(Contact)
    aor = c.build_aor(sip_request, iepg1, :inbound_end_point_host => '2.2.2.2', :inbound_end_point_port => 5060)
    aor.should == '<sip:50246380762;abc=123@2.2.2.2:5060;otg=3;npdi=yes;rn=2123334545;or_price=0.572>'
  end

  it "can build contacts with inbound rate" do
    iepg2 = iepg1.dup
    iepg2.attributes.contact_template = "<sip:${ruri.dialcode}${ruri.user_params}${oepg.uri_user_params}@${iep.host}${iep.port}${ruri.params}${oepg.uri_params};ir_price=${ir.price}>"
    route = Route.new(:inbound_rate => outbound_rate1, :outbound_end_point_group => oepg1)
    sip_request = RSIP::Message.parse(INVITE)
    a = route.contacts
    a.should be_an_instance_of(Array)
    a.length.should == 1
    c = a.first
    c.should be_an_instance_of(Contact)
    aor = c.build_aor(sip_request, iepg1, :inbound_end_point_host => '2.2.2.2', :inbound_end_point_port => 5060)
    aor.should == '<sip:50246380762;abc=123@2.2.2.2:5060;otg=3;npdi=yes;rn=2123334545;ir_price=0.572>'
  end

  context "reactor not running" do
    it "can build aors" do
      route = Route.new(:outbound_rate => outbound_rate1, :outbound_end_point_group => oepg1)
      sip_request = RSIP::Message.parse(INVITE)
      r = route.build_aors(sip_request, IEPG1)
      r.should be_an_instance_of(Array)
      route.contacts.first.aor.should be_an_instance_of(String)
      expect { aors2 = route.build_aors(sip_request, nil)  }.to raise_error
    end

    it "can build multiple aors" do
      route1 = Route.new(:outbound_rate => outbound_rate1, :outbound_end_point_group => oepg1)
      route2 = Route.new(:outbound_rate => outbound_rate2, :outbound_end_point_group => oepg2)
      sip_request = RSIP::Message.parse(INVITE)
      aors1 = route1.build_aors(sip_request, IEPG1)
      aors2 = route2.build_aors(sip_request, IEPG1)
      aors1.should == ["<sip:50246380762;abc=123@1.1.1.1:5060;otg=3;npdi=yes;rn=2123334545>"]
      aors2.should == ["<sip:50246380762@2.2.2.2:5060;otg=3;npdi=yes;rn=2123334545>"]
    end
  end

  context "reactor running" do
    it "can build aors" do
      route = Route.new(:outbound_rate => outbound_rate1, :outbound_end_point_group => oepg1)
      sip_request = RSIP::Message.parse(INVITE)
      aors = nil
      aors2 = nil

      aors = route.build_aors(sip_request, IEPG1)
      expect { aors2 = route.build_aors(sip_request, nil)  }.to raise_error

      aors.should be_an_instance_of(Array)
      route.contacts.first.aor.should be_an_instance_of(String)
    end

  end

end