require 'spec_helper'

describe RouteNGNClient::RouteTable do

  before do
    RouteNGNClient.connect(:uri => 'http://127.0.0.1')
    # RouteTables
    body = '{"total_count":1,"current_page":1,"per_page":25,"offset":0,"total_pages":1,"route_tables":[{"route_table":{"name":"Test","rate_types":["undefined","inter","intra","local"],"ordering_method":"lowest_price","max_route_depth":null,"created_at":"2013-03-05T00:12:22Z","updated_at":"2013-03-05T00:12:22Z","id":"513538666f1cc9a14000001b","account_id":"5135354e6f1cc9a140000004","outbound_end_point_group_ids":["513538546f1cc9a140000016"]}}]}'
    stub_request(:get, /^http:\/\/127\.0\.0\.1\/metallic\/route_tables\.json.*/).to_return(:body => body,
                                                                                           :status => 200,
                                                                                           :headers => { 'Content-Type' => 'application/json; charset=utf-8' })
    # RouteContainers
    body = '{"route_containers":[{"route_container":{"route_table_id":"513538666f1cc9a14000001b","rate_type":"undefined","destination_code_id":"5135425c165c882a3efc0849","inbound_rate":{"dialcode":"6","lata":null,"ocn":null,"price":0.5,"type":"undefined","weight":null,"effective_at":"2013-03-21T22:43:18Z","ineffective_at":null,"line_number":null,"minimum_duration":null,"billing_increment":null,"min_percent_price_margin":null,"min_absolute_price_margin":null,"min_route_weight":null,"id":"514b8d076f1cc973bd0000c7","inbound_end_point_group_id":"513538316f1cc9a140000010","outbound_end_point_group_id":null,"rate_sheet_id":null},"routes":[{"route":{"outbound_end_point_group_id":"513538546f1cc9a140000016","outbound_rate":{"dialcode":"6076623","lata":null,"ocn":null,"price":0.958,"type":"undefined","weight":null,"effective_at":"2013-03-05T00:12:37Z","ineffective_at":null,"line_number":27319,"minimum_duration":null,"billing_increment":null,"min_percent_price_margin":null,"min_absolute_price_margin":null,"min_route_weight":null,"id":"51353a2e6f1cc98978006b45","inbound_end_point_group_id":null,"outbound_end_point_group_id":"513538546f1cc9a140000016","rate_sheet_id":"5135388c6f1cc9a14000001f"}}}]}}]}'
    stub_request(:get, /^http:\/\/127\.0\.0\.1\/metallic\/route_tables\/513538666f1cc9a14000001b\/route_containers\.json.*/).to_return(:body => body,
                                                                                                                         :status => 200,
                                                                                                                         :headers => { 'Content-Type' => 'application/json; charset=utf-8' })

    # OEPG
    body = '{"outbound_end_point_group":{"id":"513538546f1cc9a140000016","carrier_id":"513538546f1cc9a140000017","color_code":"#f379de","created_at":"2013-03-05 00:12:04 UTC","default_route_weight":null,"destination_billing_type":"DN","dialcode_expression":"([0-9]*)$","dialcode_template":"${dialcode_match}","end_points":[{"_id":"513538546f1cc9a140000018","port":5060,"host":"1.1.1.1","priority":null}],"max_contact_end_points":null,"name":"OEPG 1","rate_link_types":["dialcode"],"route_table_ids":["513538666f1cc9a14000001b"],"updated_at":"2013-03-05 00:12:04 UTC","uri_params":"","uri_params_blacklist":[],"uri_user_params":"","uri_user_params_blacklist":[]}}'
    stub_request(:get, /^http:\/\/127\.0\.0\.1\/metallic\/outbound_end_point_groups\/513538546f1cc9a140000016\.json.*/).to_return(:body => body,
                                                                                                                         :status => 200,
                                                                                                                         :headers => { 'Content-Type' => 'application/json; charset=utf-8' })
  end

  it "can be instanstiated" do
    rt = RouteTable.new
    rt.should be_an_instance_of(RouteTable)
  end

  it "can get all via callback" do
    route_tables = nil

    RouteNGNClient::RouteTable.all({}, :account_id => 'foofoo') do |models|
      route_tables = models
    end

    route_tables.length.should == 1
    route_tables.first.should be_an_instance_of RouteNGNClient::RouteTable
  end

  it "can get all via fibers" do
    route_tables = nil

    route_tables = RouteNGNClient::RouteTable.all({}, :account_id => 'foofoo')

    route_tables.length.should == 1
    route_tables.first.should be_an_instance_of RouteNGNClient::RouteTable
  end

  it "can get route_containers" do
    route_containers = nil
    rc = nil
    e_routes = nil

    route_tables = RouteNGNClient::RouteTable.all({}, :account_id => 'foofoo')

    route_table = route_tables.first
    route_table.should be_an_instance_of RouteNGNClient::RouteTable

    params = {}
    params[:env] = { :account_id => 'foofoo' }
    params[:destination_code_id] = '5135425c165c882a3efc0849'
    params[:origination_code_id] = '5135425c165c882a3efc0849'
    params[:destination_dialcode] = '6076623'
    params[:origination_dialcode] = '6076623'
    params[:inbound_end_point_group_id] = '513538316f1cc9a140000010'
    params[:rate_type] = 'undefined'

    route_table.route_containers(params) do |models|
      route_containers = models
    end

    rc = route_containers.first
    e_routes = rc.enhanced_routes

    route_containers.length.should == 1
    rc.should be_an_instance_of RouteNGNClient::RouteContainer
    rc.routes.length.should == 1
    rc.inbound_rate.should be_an_instance_of RouteNGNClient::InboundRate
    e_routes.length.should == 1
    route = e_routes.first
    route.should be_an_instance_of RouteNGNClient::Route
    route.inbound_rate.should be_an_instance_of RouteNGNClient::InboundRate
    route.outbound_end_point_group.should be_an_instance_of RouteNGNClient::OutboundEndPointGroup
  end

end