require 'spec_helper'

describe RouteNGNClient::RemoteModel do

  before do
    RouteNGNClient.connect(:uri => 'http://127.0.0.1')

    bodya = '{"total_count":1,"current_page":1,"per_page":200,"offset":0,"total_pages":1,"testas":[{"testa":{}}]}'
    stub_request(:get, /^http:\/\/127\.0\.0\.1\/testas\.json.*/).to_return(:body => bodya,
                                                                          :status => 200,
                                                                          :headers => { 'Content-Type' => 'application/json; charset=utf-8' })

    stub_request(:get, /^http:\/\/127\.0\.0\.1\/testa_with_acct\.json\?env%5Baccount_id%5D=abcd.*/).to_return(:body => bodya,
                                                                                                              :status => 200,
                                                                                                              :headers => { 'Content-Type' => 'application/json; charset=utf-8' })

    bodyb = '{"testb":{}}'
    stub_request(:get, /^http:\/\/127\.0\.0\.1\/testb\/1\.json.*/).to_return(:body => bodyb,
                                                                             :status => 200,
                                                                             :headers => { 'Content-Type' => 'application/json; charset=utf-8' })
    end

  it "should create a new instance given valid attributes" do
    m = RemoteModel.new
    m.should be_an_instance_of(RemoteModel)
  end

  describe '#set_resource_attributes' do
    it "should set resource_attributes" do
      rm = Class.new(RemoteModel)
      rm.set_resource_attributes({:path_ext => '.foo'})
      rm.path_ext.should == '.foo'
    end
  end

  describe '#all' do
    it "should return an Array instance" do
      rm = Class.new(RemoteModel)
      rm.set_resource_attributes(:path_base => 'testas', :collection_name => 'testas')
      rm.all.should be_an_instance_of(Array)      
    end

    it "should use :account_id in options" do
      rm = Class.new(RemoteModel)
      rm.set_resource_attributes(:path_base => 'testa_with_acct', :collection_name => 'testas')
      models = rm.all({}, :account_id => 'abcd')
      models.should be_an_instance_of(Array)
      models.first.request_account_id.should == 'abcd'
    end
  end

  describe '#find' do
    it "should return an RemoteModel instance" do
      rm = Class.new(RemoteModel)
      rm.set_resource_attributes(:path_base => 'testb', :collection_name => 'testb')
      rm.find(1).should be_an_instance_of(rm)      
    end
  end

end